@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -39,12 +39,10 @@
 
 #include "debug.h"
 
+#undef  VOLUME_NORMALIZER
+#undef  SONGCHANGE_TIMEOUT
 
-#if 0
-#define OUTPUT_RATE 44100
-#else
 #define OUTPUT_RATE the_rate
-#endif
 #define OUTPUT_NCH  2
 #define OUTPUT_BPS (OUTPUT_RATE * OUTPUT_NCH * 2)
 
@@ -61,7 +59,9 @@
 #define FADE_CONFIG_STOP    4
 #define FADE_CONFIG_EOP     5
 #define FADE_CONFIG_SEEK    6
-#define MAX_FADE_CONFIGS    7
+#define FADE_CONFIG_PAUSE   7
+#define FADE_CONFIG_TIMING  8
+#define MAX_FADE_CONFIGS    9
 
 #define FADE_TYPE_REOPEN      0
 #define FADE_TYPE_FLUSH       1
@@ -71,7 +71,9 @@
 #define FADE_TYPE_ADVANCED_XF 5
 #define FADE_TYPE_FADEIN      6
 #define FADE_TYPE_FADEOUT     7
-#define MAX_FADE_TYPES        8
+#define FADE_TYPE_PAUSE_NONE  8
+#define FADE_TYPE_PAUSE_ADV   9
+#define MAX_FADE_TYPES        10
 
 #define TYPEMASK_XFADE ((1 << FADE_TYPE_REOPEN) |	\
 			(1 << FADE_TYPE_NONE) |		\
@@ -97,11 +99,14 @@
 #define TYPEMASK_EOP ((1 << FADE_TYPE_NONE) | \
 		      (1 << FADE_TYPE_FADEOUT))
 
-#define TYPEMASK_SEEK ((1 << FADE_TYPE_FLUSH) |		\
-		       (1 << FADE_TYPE_NONE) |		\
-		       (1 << FADE_TYPE_PAUSE) |		\
-		       (1 << FADE_TYPE_SIMPLE_XF) |	\
-		       (1 << FADE_TYPE_ADVANCED_XF))
+#define TYPEMASK_SEEK ((1 << FADE_TYPE_FLUSH) | \
+		       (1 << FADE_TYPE_NONE)  | \
+		       (1 << FADE_TYPE_SIMPLE_XF))
+
+#define TYPEMASK_PAUSE ((1 << FADE_TYPE_PAUSE_NONE) | \
+                        (1 << FADE_TYPE_PAUSE_ADV))
+
+#define TYPEMASK_TIMING (0)
 
 #define FC_OFFSET_NONE     0
 #define FC_OFFSET_LOCK_IN  1
@@ -117,21 +122,6 @@
 #define DEFAULT_OP_CONFIG			\
 { FALSE, FALSE, 2304, FALSE }
 
-#define DEFAULT_FADE_CONFIG			\
-{ -1,						\
-  FADE_TYPE_NONE,				\
-  2000,						\
-  4000,						\
-  FALSE, 1500, 0,				\
-  FC_OFFSET_NONE,				\
-  FC_OFFSET_NONE, -500,				\
-  TRUE,						\
-  FALSE, 1500, 0,				\
-  FALSE,					\
-  FADE,						\
-  0xffffffff					\
-}
-
 #define CONFIG_DEFAULT							\
 { 0,                            /* output_method */			\
   44100,                        /* output_rate */			\
@@ -143,12 +133,17 @@
   NULL,                         /* oss_alt_mixer_device */		\
   FALSE,                        /* oss_mixer_use_master */		\
   0,                            /* oss_buffer_size_ms */		\
-  500,                          /* oss_preload_size_ms */		\
+  250,                          /* oss_preload_size_ms */		\
+  22,                           /* oss_fragments */			\
+  12,                           /* oss_fragment_size */			\
   FALSE,                        /* oss_maxbuf_enable */			\
   NULL,                         /* op_config_string */			\
   NULL,                         /* op_name */				\
   NULL,                         /* ep_name */				\
   FALSE,                        /* ep_enable */				\
+  TRUE,                         /* volnorm_enable */			\
+  8000,                         /* volnorm_target */			\
+  FALSE,                        /* volnorm_use_qa */			\
   10000,                        /* mix_size_ms */			\
   TRUE,                         /* mix_size_auto */			\
 									\
@@ -156,24 +151,32 @@
     { FADE_CONFIG_XFADE,        /* config */				\
       FADE_TYPE_ADVANCED_XF,    /* type */				\
       2000,                     /* pause_len_ms */			\
-      3000,                     /* simple_len_ms */			\
-      TRUE, 6000, 0,            /* out_enable, _len_ms, _volume */	\
+      6000,                     /* simple_len_ms */			\
+      TRUE, 4000, 0,            /* out_enable, _len_ms, _volume */	\
       FC_OFFSET_CUSTOM,         /* ofs_type */				\
-      FC_OFFSET_CUSTOM, -8000,  /* ofs_type_wanted, ofs_custom_ms */	\
+      FC_OFFSET_CUSTOM, -6000,  /* ofs_type_wanted, ofs_custom_ms */	\
       TRUE,                     /* in_locked */				\
       FALSE, 4000, 33,          /* in_enable, _len_ms, _volume */	\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
       FALSE,                    /* flush */				\
       TYPEMASK_XFADE            /* type_mask */				\
     },									\
     { FADE_CONFIG_MANUAL,       /* config */				\
-      FADE_TYPE_FLUSH,          /* type */				\
+      FADE_TYPE_SIMPLE_XF,      /* type */				\
       2000,                     /* pause_len_ms */			\
       1000,                     /* simple_len_ms */			\
-      TRUE, 6000, 0,            /* out_enable, _len_ms, _volume */	\
+      TRUE, 500, 0,             /* out_enable, _len_ms, _volume */	\
       FC_OFFSET_CUSTOM,         /* ofs_type */				\
-      FC_OFFSET_CUSTOM, -8000,  /* ofs_type_wanted, ofs_custom_ms */	\
+      FC_OFFSET_CUSTOM, -500,   /* ofs_type_wanted, ofs_custom_ms */	\
       TRUE,                     /* in_locked */				\
-      FALSE, 4000, 33,          /* in_enable, _len_ms, _volume */	\
+      FALSE, 500, 50,           /* in_enable, _len_ms, _volume */	\
+      FALSE,                    /* flush_pause_enable */		\
+      500,                      /* flush_in_len_ms */			\
+      FALSE,                    /* flush_in_enable */			\
+      500, 0,                   /* flush_in_len_ms, _volume */		\
       TRUE,                     /* flush */				\
       TYPEMASK_MANUAL           /* type_mask */				\
     },									\
@@ -187,10 +190,14 @@
       FALSE,                    /* - */					\
       FALSE, 1000, 0,           /* - */					\
       FALSE,                    /* - */					\
-      TYPEMASK_ALBUM            /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
+      FALSE,                    /* flush */				\
+      TYPEMASK_ALBUM            /* type_mask */				\
     },									\
     { FADE_CONFIG_START,        /* config */				\
-      FADE_TYPE_NONE,           /* type */				\
+      FADE_TYPE_FADEIN,         /* type */				\
       0,                        /* - */					\
       0,                        /* - */					\
       FALSE, 0, 0,              /* - */					\
@@ -198,11 +205,15 @@
       FC_OFFSET_NONE, 0,        /* - */					\
       FALSE,                    /* - */					\
       FALSE, 100, 0,            /* - in_len_ms, _volume */		\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
       TRUE,                     /* flush */				\
       TYPEMASK_START            /* type_mask */				\
     },									\
     { FADE_CONFIG_STOP,         /* config */				\
-      FADE_TYPE_NONE,           /* type */				\
+      FADE_TYPE_FADEOUT,        /* type */				\
       0,                        /* - */					\
       0,                        /* - */					\
       FALSE, 100, 0,            /* - out_len_ms, _volume */		\
@@ -210,11 +221,15 @@
       FC_OFFSET_NONE, 500,      /* - ofs_custom_ms */			\
       FALSE,                    /* - */					\
       FALSE, 0, 0,              /* - */					\
-      TRUE,                     /* - */					\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
+      TRUE,                     /* flush */				\
       TYPEMASK_STOP             /* type_mask */				\
     },									\
     { FADE_CONFIG_EOP,          /* config */				\
-      FADE_TYPE_NONE,           /* type */				\
+      FADE_TYPE_FADEOUT,        /* type */				\
       0,                        /* - */					\
       0,                        /* - */					\
       FALSE, 100, 0,            /* - out_len_ms, _volume */		\
@@ -222,20 +237,60 @@
       FC_OFFSET_NONE, 500,      /* - ofs_custom_ms  */			\
       FALSE,                    /* - */					\
       FALSE, 0, 0,              /* - */					\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
       FALSE,                    /* flush */				\
       TYPEMASK_EOP              /* type_mask */				\
     },									\
     { FADE_CONFIG_SEEK,         /* config */				\
-      FADE_TYPE_FLUSH,          /* type */				\
-      500,                      /* pause_len_ms */			\
+      FADE_TYPE_SIMPLE_XF,      /* type */				\
+      0,                        /* - */					\
       50,                       /* simple_len_ms */			\
-      TRUE, 6000, 0,            /* out_enable, _len_ms, _volume */	\
-      FC_OFFSET_CUSTOM,         /* ofs_type */				\
-      FC_OFFSET_CUSTOM, -8000,  /* ofs_type_wanted, ofs_custom_ms */	\
-      TRUE,                     /* in_locked */				\
-      FALSE, 4000, 33,          /* in_enable, _len_ms, _volume */	\
+      FALSE, 0, 0,              /* - */					\
+      FC_OFFSET_NONE,           /* - */					\
+      FC_OFFSET_NONE, 0,        /* - */					\
+      FALSE,                    /* - */					\
+      FALSE, 1000, 0,           /* - */					\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
       TRUE,                     /* flush */				\
       TYPEMASK_SEEK             /* type_mask */				\
+    },									\
+    { FADE_CONFIG_PAUSE,        /* config */				\
+      FADE_TYPE_PAUSE_ADV,      /* type */				\
+      0,                        /* - */					\
+      0,                        /* - */					\
+      TRUE, 100, 0,             /* - out_len_ms, - */			\
+      FC_OFFSET_NONE,           /* - */					\
+      FC_OFFSET_NONE, 100,      /* - ofs_custom_ms */			\
+      FALSE,                    /* - */					\
+      TRUE, 100, 0,             /* - in_len_ms, - */			\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
+      FALSE,                    /* - */					\
+      TYPEMASK_PAUSE            /* type_mask */				\
+    },									\
+    { FADE_CONFIG_TIMING,       /* config */				\
+      FADE_TYPE_NONE,           /* type */				\
+      0,                        /* - */					\
+      0,                        /* - */					\
+      TRUE, 0, 0,               /* out_enable, _len_ms, - */		\
+      FC_OFFSET_CUSTOM,         /* - */					\
+      FC_OFFSET_CUSTOM, 0,      /* - ofs_custom_ms */			\
+      FALSE,                    /* - */					\
+      TRUE, 0, 0,               /* in_enable, _len_ms, - */		\
+      FALSE,                    /* - */					\
+      0,                        /* - */					\
+      FALSE,                    /* - */					\
+      0, 0,                     /* - */					\
+      FALSE,                    /* flush */				\
+      TYPEMASK_TIMING           /* type_mask */				\
     }									\
   },									\
   TRUE,                         /* gap_lead_enable */			\
@@ -245,16 +300,23 @@
   500,                          /* gap_trail_len_ms */			\
   512,                          /* gap_trail_level */			\
   TRUE,                         /* gap_trail_locked */			\
+  TRUE,                         /* gap_crossing */			\
   FALSE,                        /* enable_debug */			\
   FALSE,                        /* enable_monitor */			\
   TRUE,                         /* enable_mixer */			\
   FALSE,                        /* mixer_reverse */			\
+  FALSE,                        /* mixer_software */			\
+  75,                           /* mixer_vol_left */			\
+  75,                           /* mixer_vol_right */			\
   500,                          /* songchange_timeout */		\
   0,                            /* preload_size_ms */			\
   TRUE,                         /* album detection */			\
+  FALSE,                        /* no_xfade_if_same_file */		\
   FALSE,                        /* enable_http_workaround */		\
   FALSE,                        /* enable_op_max_used */		\
   250,                          /* op_max_used_ms */			\
+  FALSE,                        /* output_keep_opened */		\
+  NULL,                         /* presets */				\
   250                           /* sync_size_ms */			\
 }
 
@@ -268,13 +330,15 @@
 
 
 /* get plugin info (imported by XMMS) */
-OutputPlugin *get_oplugin_info(void);
+OutputPlugin *get_oplugin_info();
 
 /* xmms internal prototypes */ /* XMMS */
-gint   ctrlsocket_get_session_id(void);
-GList *get_output_list(void);
-GList *get_effect_list(void);
-gint   get_playlist_position(void);
+gint     ctrlsocket_get_session_id();
+gboolean get_input_playing();
+GList   *get_output_list();
+GList   *get_effect_list();
+gint     get_playlist_position();
+gint     playlist_get_current_length();
 #ifdef HAVE_PLAYLIST_GET_FILENAME
 gchar *playlist_get_filename(gint pos);
 #endif
@@ -309,10 +373,14 @@ typedef struct
   gint   fade_len;      /* length of fadein */
   gfloat fade_scale;    /* 1.0f - in_level */
 
+#define GAP_SKIPPING_POSITIVE -1
+#define GAP_SKIPPING_NEGATIVE -2
+#define GAP_SKIPPING_DONE     -3
   gint   gap;           /* > 0: removing (leading) gap */
   gint   gap_len;       /* max. len of gap, 0=disabled */
   gint   gap_level;     /* max. sample value+1 to be considered "silent" */
   gint   gap_killed;    /* number of bytes that were killed last time */
+  gint   gap_skipped;   /* negative/positive samples skipped */
 
   /* ---------------------------------------------------------------------- */
 
@@ -328,28 +396,35 @@ buffer_t;
 
 typedef struct
 {
-  gint     config;
-  gint     type;
+  gint     config;              // one of FADE_CONFIG_*, constant
+  gint     type;                // one of FADE_TYPE_*
 
-  gint     pause_len_ms;     // PAUSE
+  gint     pause_len_ms;        // PAUSE
 
-  gint     simple_len_ms;    // SIMPLE_XF
+  gint     simple_len_ms;       // SIMPLE_XF
 
-  gboolean out_enable;       // ADVANCED_XF
-  gint     out_len_ms;       // ADVANCED_XF, FADEOUT
-  gint     out_volume;       // ADVANCED_XF, FADEOUT
+  gboolean out_enable;          // ADVANCED_XF
+  gint     out_len_ms;          // ADVANCED_XF, FADEOUT, PAUSE
+  gint     out_volume;          // ADVANCED_XF, FADEOUT
 
-  gint     ofs_type;         // ADVANCED_XF
-  gint     ofs_type_wanted;  // ADVANCED_XF
-  gint     ofs_custom_ms;    // ADVANCED_XF, FADEOUT (additional silence)
+  gint     ofs_type;            // ADVANCED_XF
+  gint     ofs_type_wanted;     // ADVANCED_XF
+  gint     ofs_custom_ms;       // ADVANCED_XF, FADEOUT (additional silence), PAUSE
 
-  gboolean in_locked;        // ADVANCED_XF
-  gboolean in_enable;        // ADVANCED_XF
-  gint     in_len_ms;        // ADVANCED_XF, FADEIN
-  gint     in_volume;        // ADVANCED_XF, FADEIN
+  gboolean in_locked;           // ADVANCED_XF
+  gboolean in_enable;           // ADVANCED_XF, FLUSH
+  gint     in_len_ms;           // ADVANCED_XF, FLUSH, FADEIN, PAUSE
+  gint     in_volume;           // ADVANCED_XF, FLUSH, FADEIN
 
-  gboolean flush;            // TRUE for manual, FALSE for xfade config
-  guint32  type_mask;        // bitmask for FADE_TYPEs
+  gboolean flush_pause_enable;  // FLUSH
+  gint     flush_pause_len_ms;  // FLUSH
+  gboolean flush_in_enable;     // FLUSH
+  gint     flush_in_len_ms;     // FLUSH
+  gint     flush_in_volume;     // FLUSH
+
+  /* additional stuff which is not configureable by the user */
+  gboolean flush;               // TRUE for manual, FALSE for xfade config
+  guint32  type_mask;           // bitmask for FADE_TYPEs
 }
 fade_config_t;
 
@@ -380,6 +455,9 @@ typedef struct
 
   gint     oss_buffer_size_ms;
   gint     oss_preload_size_ms;
+
+  gint     oss_fragments;
+  gint     oss_fragment_size;
   gboolean oss_maxbuf_enable;
 
   /* output method: plugin */
@@ -389,6 +467,9 @@ typedef struct
   /* effects */
   gchar   *ep_name;
   gboolean ep_enable;
+  gboolean volnorm_enable;
+  gboolean volnorm_use_qa;
+  gint     volnorm_target;
 
   /* crossfader */
   gint          mix_size_ms;
@@ -399,24 +480,35 @@ typedef struct
   gboolean gap_lead_enable;
   gint     gap_lead_len_ms;
   gint     gap_lead_level;
+
   gboolean gap_trail_enable;
   gint     gap_trail_len_ms;
   gint     gap_trail_level;
   gboolean gap_trail_locked;
 
+  gboolean gap_crossing;
+
   /* misc */
   gboolean enable_debug;
   gboolean enable_monitor;
 
   gboolean enable_mixer;
   gboolean mixer_reverse;
+  gboolean mixer_software;
+  gint     mixer_vol_left;
+  gint     mixer_vol_right;
 
   gint     songchange_timeout;
   gint     preload_size_ms;
   gboolean album_detection;
+  gboolean no_xfade_if_same_file;
   gboolean enable_http_workaround;
   gboolean enable_op_max_used;
   gint     op_max_used_ms;
+  gboolean output_keep_opened;
+
+  /* presets */
+  GList   *presets;
 
   /* additional stuff which is not configureable by the user */
   gint     sync_size_ms;
@@ -438,6 +530,8 @@ extern pthread_mutex_t buffer_mutex;
 
 extern gboolean opened;         /* XMMS-crossfade is opened */
 extern gboolean output_opened;  /* the output plugin is opened */
+extern gint     output_offset;
+extern gint64   output_streampos;
 
 extern OutputPlugin *the_op;
 extern gint          the_rate;
