@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -26,6 +26,9 @@
 
 #include <glib.h>
 
+/* time between updates in ms (default: 33 -> 30 updates per second) */
+#define UPDATE_INTERVAL 33
+
 void xfade_check_monitor_win();
 gint xfade_update_monitor   (gpointer userdata);
 void xfade_start_monitor    ();
