@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -31,6 +31,7 @@
 #include "oss.h"
 
 #include <stdio.h>
+#include <string.h>
 
 #include <fcntl.h>
 #include <unistd.h>
@@ -89,6 +90,7 @@ static gboolean realtime;
 static gint     dsp_fd = -1;
 static gint     dsp_buffer_size = 0;
 static gint     dsp_fragment_size = 0;
+static gint     dsp_fragment_utime = 0;
 static gboolean dsp_select_works = FALSE;
 
 static pthread_mutex_t buffer_mutex;
@@ -112,13 +114,23 @@ static gint      buffer_preload_size;
 
 static effect_context_t effect_context;
 
-
 OutputPlugin *
 xfade_get_builtin_oss_oplugin_info(void)
 {
   return &oss_op;
 }
 
+static char *
+get_mixer_device_name()
+{
+  if(config->oss_use_alt_mixer_device)
+    return g_strdup(config->oss_alt_mixer_device);
+  else if(config->oss_mixer_device > 0)
+    return g_strdup_printf("/dev/mixer%d", config->oss_mixer_device);
+  else
+    return g_strdup("/dev/mixer");
+}
+
 void
 oss_get_volume(int *l, int *r)
 {
@@ -133,10 +145,7 @@ oss_get_volume(int *l, int *r)
   if(!config->enable_mixer) return;
 
   /* get device name */
-  if(config->oss_mixer_device > 0)
-    mixer_device_name = g_strdup_printf("/dev/mixer%d", config->oss_mixer_device);
-  else
-    mixer_device_name = g_strdup("/dev/mixer");
+  mixer_device_name = get_mixer_device_name();
 
   /* open device */
   mixer_fd = open(mixer_device_name, O_RDONLY);
@@ -180,10 +189,7 @@ oss_set_volume(int l, int r)
   if(!config->enable_mixer) return;
 
   /* get device name */
-  if(config->oss_mixer_device > 0)
-    mixer_device_name = g_strdup_printf("/dev/mixer%d", config->oss_mixer_device);
-  else
-    mixer_device_name = g_strdup("/dev/mixer");
+  mixer_device_name = get_mixer_device_name();
 
   /* open device */
   mixer_fd = open(mixer_device_name, O_RDONLY);
@@ -226,11 +232,13 @@ oss_open_audio(AFormat fmt, int rate, int nch)
   struct timeval tv;
   fd_set         set;
 
+  DEBUG(("[xfade-oss] open_audio: pid=%d\n", getpid()));
+
   /* check if we should use as much device buffer as available */
   if(config->oss_maxbuf_enable)
     dsp_fragment = OSS_MAXFRAGMENT;
   else
-    dsp_fragment = OSS_FRAGMENT;
+    dsp_fragment = config->oss_fragments << 16 | config->oss_fragment_size;
 
   /* check for realtime priority, it needs some special attention */
   realtime = xmms_check_realtime_priority();
@@ -315,18 +323,18 @@ oss_open_audio(AFormat fmt, int rate, int nch)
    *       locks the device between the close() and open() calls below. */
   dsp_fd = open(device_name, O_WRONLY|O_NONBLOCK);
   if(dsp_fd == -1) {
-    PERROR("[xfade-oss] oss_open_audio");
+    PERROR("[xfade-oss] open_audio");
     g_free(device_name);
     return 0;
   }
   close(dsp_fd);
 
-  /* open device */
+  /* open device (we do need blocking io here!)  */
   dsp_fd = open(device_name, O_WRONLY);
   g_free(device_name);
   
   if(dsp_fd == -1) {
-    PERROR("[xfade-oss] oss_open_audio");
+    PERROR("[xfade-oss] open_audio");
     return 0;
   }
   
@@ -339,10 +347,11 @@ oss_open_audio(AFormat fmt, int rate, int nch)
     dsp_fd = -1;
     return 0;
   }
-							  
+
   /* set fragment size/count */
-  if(ioctl(dsp_fd, SNDCTL_DSP_SETFRAGMENT, &dsp_fragment) == -1)
-    PERROR("[xfade-oss] open_audio: ioctl(SETFRAGMENT)");
+  if(!config->oss_maxbuf_enable)
+    if(ioctl(dsp_fd, SNDCTL_DSP_SETFRAGMENT, &dsp_fragment) == -1)
+      PERROR("[xfade-oss] open_audio: ioctl(SETFRAGMENT)");
   
   /* get dsp_buffer_size */
   if(ioctl(dsp_fd, SNDCTL_DSP_GETOSPACE, &info) == -1) {
@@ -351,9 +360,10 @@ oss_open_audio(AFormat fmt, int rate, int nch)
     dsp_fd = -1;
     return 0;
   }
-  dsp_buffer_size   = info.fragsize * info.fragstotal;
-  dsp_fragment_size = info.fragsize;
-  
+  dsp_buffer_size    = info.fragsize * info.fragstotal;
+  dsp_fragment_size  = info.fragsize;
+  dsp_fragment_utime = (gint)((gint64)dsp_fragment_size * (1000000/2/2) / OUTPUT_RATE);
+
   /* HACK: (from OSS/audio.c) find out if the driver supports selects */
   tv.tv_sec = 0;
   tv.tv_usec = 50000;
@@ -371,17 +381,19 @@ oss_open_audio(AFormat fmt, int rate, int nch)
   DEBUG(("[xfade-oss] open_audio: device: fragsize=%d bytes=%d (%d ms)\n",
 	 info.fragsize, info.bytes, B2MS(info.bytes)));
 
-
   /* calculate buffer size */
   buffer_buffer_size  = MS2B(config->oss_buffer_size_ms)  & -4;
   buffer_preload_size = MS2B(config->oss_preload_size_ms) & -4;
   buffer_size         = buffer_buffer_size + buffer_preload_size;
 
-  if(buffer_size < MIN_BUFFER_SIZE)
-    buffer_size = MIN_BUFFER_SIZE;
+  if(buffer_size < dsp_fragment_size) {
+    DEBUG(("[xfade-oss] open_audio: buffer size adjusted to match fragment size\n"));
+    buffer_preload_size += dsp_fragment_size - buffer_size;
+    buffer_size = dsp_fragment_size;
+  }
 
   buffer_size += dsp_buffer_size;  /* for pause rewind */
-  
+
   /* allocate buffer */
   if(!(buffer_data = g_malloc0(buffer_size))) {
     DEBUG(("[xfade-oss] open_audio: error allocating %d bytes "
@@ -390,14 +402,15 @@ oss_open_audio(AFormat fmt, int rate, int nch)
     dsp_fd = -1;
     return 0;
   }
-  
+
   DEBUG(("[xfade-oss] open_audio: buffer: size=%d (%d+%d+%d=%d ms)\n",
 	 buffer_size,
 	 B2MS(buffer_buffer_size),
 	 B2MS(buffer_preload_size),
 	 B2MS(dsp_buffer_size),
 	 B2MS(buffer_size)));
-  
+
+
   /* reset buffer */
   streampos       = 0;
   buffer_rd_index = 0;
@@ -416,8 +429,7 @@ oss_open_audio(AFormat fmt, int rate, int nch)
   /* create and run buffer thread (joinable) */
   if(pthread_create(&buffer_thread, NULL, buffer_thread_f, NULL)) {
     PERROR("[xfade-oss] open_audio: pthread_create()");
-    g_free(buffer_data);
-    buffer_data = NULL;
+    g_free(buffer_data); buffer_data = NULL;
     close(dsp_fd);
     dsp_fd = -1;
     return 0;
@@ -443,10 +455,10 @@ oss_write_audio(void *ptr, int length)
   /* lock buffer */
   pthread_mutex_lock(&buffer_mutex);
   
-  /* print warning on buffer overrun (this is really bad!) */
+  /* print warning on buffer overrun */
   free = buffer_size - buffer_used;
   if(length > free) {
-    DEBUG(("[xfade-oss] write_audio: %d bytes truncated!\n", length-free));
+    DEBUG(("[xfade-oss] write_audio: WARNING: %d bytes truncated!\n", length-free));
     length = free;
   }
 
@@ -492,6 +504,7 @@ buffer_thread_f(void *arg)
   struct timeval tv;
  
   DEBUG(("[xfade-oss] buffer_thread_f: thread started\n"));
+  DEBUG(("[xfade-oss]\n"));
   
   /* lock buffer */
   pthread_mutex_lock(&buffer_mutex);
@@ -500,7 +513,7 @@ buffer_thread_f(void *arg)
     /* wait for device */
     if(dsp_select_works) {
       tv.tv_sec  = 0;
-      tv.tv_usec = OSS_FRAGMENT_UTIME;
+      tv.tv_usec = dsp_fragment_utime;
       FD_ZERO(&set);
       FD_SET(dsp_fd, &set);
 
@@ -516,19 +529,30 @@ buffer_thread_f(void *arg)
     }
     else {
       pthread_mutex_unlock(&buffer_mutex);
-      xmms_usleep(OSS_FRAGMENT_UTIME);
+      xmms_usleep(dsp_fragment_utime);
       pthread_mutex_lock(&buffer_mutex);
     }
 
     /* get free space in device output buffer */
     if(ioctl(dsp_fd, SNDCTL_DSP_GETOSPACE, &info)) {
       PERROR("[xfade-oss] buffer_thread_f: ioctl(GETOSPACE)");
-      break;
+
+      /* 0.3.4: try to reset device */
+      if(ioctl(dsp_fd, SNDCTL_DSP_RESET, 0)) {
+	PERROR("[xfade-oss] buffer_thread_f: ioctl(RESET)");
+	break;  /* give up */
+      }
+      continue;
     }
 
     /* continue waiting if there is no room in the device buffer */
     if(info.bytes < 4) continue;
 
+    /* check for buffer underrun */
+    if(info.fragments == info.fragstotal) {
+      /* FIXME: maybe do some sensible recovering */
+    }
+
     /* is there any data in the buffer? */
     if(!paused && (buffer_used >= 4) && (buffer_preload <= 0)) {
       /* write as much data as a) is available and b) the device can take */
@@ -555,11 +579,11 @@ buffer_thread_f(void *arg)
       }
     }
     else if(dsp_select_works) {
-      /* HACK: Force yielding. This is _necessary_ in realtime mode.
+      /* HACK: Force yielding. This is _vital_ in realtime mode.
        *       Without it this thread would eat up all CPU time polling
        *       the audio device with select(), having no samples to write. */
       pthread_mutex_unlock(&buffer_mutex);
-      xmms_usleep(OSS_FRAGMENT_UTIME);
+      xmms_usleep(dsp_fragment_utime/4);
       pthread_mutex_lock(&buffer_mutex);
     }
   }
@@ -576,6 +600,10 @@ void oss_close_audio()
   /* lock buffer */
   pthread_mutex_lock(&buffer_mutex);
 
+  /* free buffer */
+  g_free(buffer_data);
+
+  /* close device */
   if(dsp_fd != -1) {
     ioctl(dsp_fd, SNDCTL_DSP_RESET, NULL);
     close(dsp_fd);
@@ -638,13 +666,17 @@ void oss_pause(short p)
     /* don't rewind too far if playback just has started */
     if(rewind > buffer_written) rewind = buffer_written;
 
+    /* make sure we always operate on sample boundaries */
+    /* NOTE: this one does happen quite often */
+    rewind &= -4;
+
     /* sanity check (should never happen) */
     if(rewind > (buffer_size - buffer_used)) {
       DEBUG(("[xfade-oss] pause: rewind too large (%d, free=%d)!\n",
 	     rewind, buffer_size - buffer_used));
       rewind = buffer_size - buffer_used;
     }
-    
+
     /* reset device, this will stop playback immediatelly */
     ioctl(dsp_fd, SNDCTL_DSP_RESET, NULL);
 
@@ -695,7 +727,7 @@ gint oss_buffer_playing()
   }
   
   dsp_used = dsp_buffer_size - info.bytes;
-  playing  = (buffer_used > 0) || (dsp_used >= dsp_fragment_size);
+  playing  = (buffer_used > 0) || (dsp_used >= 3*dsp_fragment_size);
 
   /* unlock buffer */
   pthread_mutex_unlock(&buffer_mutex);
@@ -715,7 +747,7 @@ gint oss_output_time()
     pthread_mutex_unlock(&buffer_mutex);
     return 0;
   }
-  
+
   time = oss_written_time() - B2MS(buffer_used + (dsp_buffer_size-info.bytes));
   if(time < 0) time = 0;
 
