@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2001  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -36,13 +36,7 @@
 #  include <machine/soundcard.h>
 #endif
 
-/* device buffer size: 22*2048 ~= 250 ms */
-#define OSS_FRAGMENT    0x0016000b
 #define OSS_MAXFRAGMENT 0xffff000b
-#define OSS_FRAGMENT_SIZE 2048
-#define OSS_FRAGMENT_UTIME (OSS_FRAGMENT_SIZE*1000000/OUTPUT_BPS)
-
-#define MIN_BUFFER_SIZE 2048
 
 /* get plugin info */
 OutputPlugin *xfade_get_builtin_oss_oplugin_info(void);
